﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace BoEBot_Commander
{
    public partial class BoESerialPort : UserControl
    {
        //DECLARATIONS FOR CUSTOM EVENT
        public event MessageReceivedHandler MessageReceived;
        public EventArgs e = null;
        public delegate void MessageReceivedHandler(BoESerialPort b, EventArgs e);

        //DECLARATIONS FOR OUT OF SYNC DELEGATE TO HANDLE SERIAL COMMS
        public delegate void ProcessDataInDelegate(byte[] baIn);
        public ProcessDataInDelegate ProcessDataInHandler;

        //----------------------------
        //GLOBAL VARIABLE DECLARATIONS
        //----------------------------
        private string g_strMessage;
        private LinkedList <string> g_listMessageQueue;
        private bool g_bCaptureMessage;
        private bool g_bIsOpen = false;
        private int g_iPort = 1;
        private int g_iBaudRate = 9600;
        private int g_iDataBits = 8;
        //----------------------------

        // ** PROPERTIES **
        //public string Message { get { return g_strMessage; } }
        public int Port { get { return g_iPort; } set { g_iPort = value; } }
        public int BaudRate { get { return g_iBaudRate; } set { g_iBaudRate = value; } }
        public int DataBits { get { return g_iDataBits; } set { g_iDataBits = value; } }
        public bool IsOpen { get { return g_bIsOpen; } }

        
        /// <summary>
        /// 
        /// </summary>
        public BoESerialPort()
        {
            InitializeComponent();
            ProcessDataInHandler = new ProcessDataInDelegate(ProcessDataIn);

            g_strMessage = "";
            g_bCaptureMessage = false;
            g_listMessageQueue = new LinkedList<string>();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="baIn"></param>
        private void ProcessDataIn(byte[] baIn)
        {
            //---------------------------
            //LOCAL VARIABLE DECLARATIONS
            //---------------------------
            //---------------------------

            //IF WE HAVE A FULL MESSAGE RAISE OUR CUSTOM EVENT
            // - MessageRecieved (this, e);

            //RUN THROUGH THE BYTES
            for (int iLCV = 0; iLCV < baIn.Length; iLCV++)
            {
                //CHECK IF WE ARE CAPTURING A MESSAGE
                if (g_bCaptureMessage)
                {
                    //CHECK FOR END OF MESSAGE
                    if (baIn[iLCV] == '~')
                    {
                        //DONE
                        g_bCaptureMessage = false;
                        //QUEUE IT UP
                        g_listMessageQueue.AddLast(g_strMessage);
                        g_strMessage = "";
                        //RAISE THE EVENT
                        MessageReceived(this, e);
                    }
                    else
                    {
                        g_strMessage += Convert.ToChar (baIn[iLCV]);
                    }
                }
                else
                {
                    //CHECK FOR START MESSAGE
                    if (baIn[iLCV] == '~')
                        g_bCaptureMessage = true;
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public bool Open()
        {
            try
            {
                portCOM.PortName = "COM" + Port;
                portCOM.BaudRate = BaudRate;
                portCOM.DataBits = DataBits;
                portCOM.Open();
                this.BackColor = Color.Green;
                g_bIsOpen = true;
                return true;
            }
            catch
            {
                return false;
            }
        }

        public bool Close()
        {
            try
            {
                portCOM.Close();
                this.BackColor = Color.Red;
                g_bIsOpen = false;
                return true;
            }
            catch
            {
                return false;
            }
        }

        public byte[] BuildMessage(byte btCommand, byte btParam1, byte btParam2)
        {
            byte[] baTemp = new byte[5];
            baTemp[0] = 255;
            baTemp[1] = btCommand;
            baTemp[2] = btParam1;
            baTemp[3] = btParam2;
            baTemp[4] = 0;
            return baTemp;
        }

        public void PostMessage(byte[] baMessage)
        {
            portCOM.Write(baMessage, 0, 5);
        }

        public string GetMessage()
        {
            string strMessage = "";
            LinkedListNode <string> nodeMessage = g_listMessageQueue.First;
            if (nodeMessage != null)
            {
                strMessage = nodeMessage.Value;
                g_listMessageQueue.Remove(nodeMessage);
            }else
            {
                strMessage = "";
            }
            return strMessage;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void portCOM_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            //---------------------------
            //LOCAL VARIABLE DECLARATIONS
            //---------------------------
            int iNumBytes;
            byte[] baIn;
            //---------------------------

            try
            {
                //GET NUMBER OF BYTES TO READ
                iNumBytes = portCOM.BytesToRead;
                //READ SAID BYTES INTO A BYTE ARRAY
                baIn = new byte[iNumBytes];
                portCOM.Read(baIn, 0, iNumBytes);
                //INVOKE THE ASYNCH DELEGATE TO DEAL WITH THE DATA
                this.Invoke(ProcessDataInHandler, baIn);
            }
            catch
            {
            }
        }

    }
}
